#include "JCSVFileReaderObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJCSVFileReaderObj = JCSVFileReaderObj().Register();


void JCSVFileReaderObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "file", file);
}

void JCSVFileReaderObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_file = getObject( dict, "file");
  if ( obj_file ) file = *(JString*)obj_file;
}

const char* JCSVFileReaderObj::className() const { return "JCSVFileReaderObj";}


JObject* JCSVFileReaderObj::clone() const { return new JCSVFileReaderObj(*this);}


// class constructor
JCSVFileReaderObj::JCSVFileReaderObj() {
  igm = 1 + 1 + 1;
  ogm = MAX_OUTPUTS;
  on = 0;
  tick = 0;
  numberRecords = 0;
  for ( int i = 0; i < MAX_OUTPUTS; i++ ) {
	  rout[i] = 0;
  }
  r0 = 0;
  inStreamAvailable = 0;
}


void JCSVFileReaderObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("CSVreader"), rect);
}


boolean JCSVFileReaderObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JCSVFileReaderObj::inputTag(int n) {
  char* tag[] = {
    "on",
    "tick",
    "inputFile" };
  return( tag[n] );
}


JString JCSVFileReaderObj::outputTag(int n) {
  char* tag[] = {
    "r0",
    "r1",
    "r2",
    "r3",
    "r4",
    "r5",
    "r6",
    "r7",
    "r8",
    "r9",
    "r10",
	"r11",
	"r12",
	"r13",
	"r14",
	"r15",
	"r16",
	"r17",
	"r18",
	"r19",
	"r20",
	"r21",
	"r22",
	"r23",
	"r24",
	"r25",
	"r26",
	"r27",
	"r28",
	"r29",
	"r30",
	"r31",
	"r32",
	"r33",
	"r34",
	"r35",
	"r36",
	"r37",
	"r38",
	"r39",
	"r40"
  };
  return( tag[n] );
}

int JCSVFileReaderObj::inputType( int n ) {
  if ( n != 2) return JIntegerData::id;
  return JStringData::id;
}

JFRect JCSVFileReaderObj::inputArea(int n) {
  return topArea( n, 0, 1 + 1 + 1, 0.25 );
}


JFRect JCSVFileReaderObj::outputArea(int n) {
  return rightArea(n, 0, 1 + MAX_OUTPUTS);
}


int JCSVFileReaderObj::inputFace(int n) {
  return TOP;
}


int JCSVFileReaderObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JCSVFileReaderObj::inputPoint(int n, JLinkObj& link) {
  return topPoint(n, link, 0, 1 + 1 + 1 );
}


JFPoint JCSVFileReaderObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1 + MAX_OUTPUTS);
}


void JCSVFileReaderObj::access(int n, JLinkObj& link, const JDataType& data) {
  INT(data) = rout[n];
}

#include <ctype.h>
char* grabInt( char* scanner, int* xresult ) {
	int result = 0;
	if ( scanner == NULL ) {
		return( NULL );
	}
	while ( *scanner ) {
		if ( isdigit( *scanner )) {
			result = result*10 + (*scanner - '0');
		} else {
			scanner++;
			*xresult = result;
			return( scanner );
		}
		scanner++;
	}
	return( NULL );
}

void JCSVFileReaderObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case ON:
    link.access(JIntegerData(on));
	if ( on ) {
	  JComponent::debug("reader is on");
		inFile = JFile( file );
		if ( inFile.exists() ) {
			JComponent::debug( "File exists" );
			inputStream = fopen( file, "r" );
			if ( inputStream == NULL ) {
				inStreamAvailable = 0;
			} else {
				inStreamAvailable = 1;
				JComponent::debug("input stream is available");
			}
		} else {
			JComponent::debug( "File does not exist" );
			inStreamAvailable = 0;
		}
	} else {
	  JComponent::debug("reader is off");
	}
    break;
  case TICK:
    link.access(JIntegerData(tick));
	JComponent::debug("got tick");
	if ( tick ) {
		if ( inStreamAvailable ) {
			char buf[1500];
			if ( fgets( &buf[0], 1499, inputStream ) == NULL ) {
				inStreamAvailable = 0;
				fclose( inputStream );
			} else {
				JComponent::debug("got data");
				char* scanner = &buf[0];
				for ( int i = 0; i < MAX_OUTPUTS; i++ ) {
					scanner = grabInt( scanner, &rout[i] );
					if ( scanner ) broadcast( i );
					else break;
				}
			}

			//JString x;
			//inputStream >> x;
			//JComponent::debug( "test" );
		}
	}
    break;
  case NUMBERRECORDS:
    link.access(JStringData(file));
    break;
  default:
    break;
  }
}


JArray JCSVFileReaderObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("file", file));
  return properties;
}


boolean JCSVFileReaderObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("file")) {
    file = ((JStringProperty*)&prop)->value;
    return true;
  }
  return false;
}


